@@ -26,6 +26,7 @@ from nova import utils
 from nova import test
 from nova.compute import manager
 from nova.compute import power_state
+from nova.compute import vm_states
 from nova.network import model as network_model
 from nova.tests.unit import fake_instance
 from pylxd import exceptions as lxdcore_exceptions
@@ -203,10 +204,6 @@ class LXDDriverTest(test.NoDBTestCase):
         info = lxd_driver.get_info(instance)
 
         self.assertEqual(power_state.RUNNING, info.state)
-        self.assertEqual(3, info.mem_kb)
-        self.assertEqual(4, info.max_mem_kb)
-        self.assertEqual(1, info.num_cpu)
-        self.assertEqual(0, info.cpu_time_ns)
 
     def test_list_instances(self):
         self.client.containers.all.return_value = [
@@ -261,6 +258,7 @@ class LXDDriverTest(test.NoDBTestCase):
         image_meta = mock.Mock()
         injected_files = mock.Mock()
         admin_password = mock.Mock()
+        allocations = mock.Mock()
         network_info = [_VIF]
         block_device_info = mock.Mock()
         virtapi = manager.ComputeVirtAPI(mock.MagicMock())
@@ -274,7 +272,7 @@ class LXDDriverTest(test.NoDBTestCase):
 
         lxd_driver.spawn(
             ctx, instance, image_meta, injected_files, admin_password,
-            network_info, block_device_info)
+            allocations, network_info, block_device_info)
 
         self.vif_driver.plug.assert_called_once_with(
             instance, network_info[0])
@@ -293,6 +291,7 @@ class LXDDriverTest(test.NoDBTestCase):
         image_meta = mock.Mock()
         injected_files = mock.Mock()
         admin_password = mock.Mock()
+        allocations = mock.Mock()
 
         lxd_driver = driver.LXDDriver(None)
         lxd_driver.init_host(None)
@@ -302,7 +301,7 @@ class LXDDriverTest(test.NoDBTestCase):
 
             lxd_driver.spawn,
             ctx, instance, image_meta, injected_files, admin_password,
-            None, None)
+            allocations, None, None)
 
     @mock.patch('nova.virt.configdrive.required_by')
     def test_spawn_with_configdrive(self, configdrive):
@@ -318,6 +317,7 @@ class LXDDriverTest(test.NoDBTestCase):
         image_meta = mock.Mock()
         injected_files = mock.Mock()
         admin_password = mock.Mock()
+        allocations = mock.Mock()
         network_info = [_VIF]
         block_device_info = mock.Mock()
         virtapi = manager.ComputeVirtAPI(mock.MagicMock())
@@ -332,7 +332,7 @@ class LXDDriverTest(test.NoDBTestCase):
 
         lxd_driver.spawn(
             ctx, instance, image_meta, injected_files, admin_password,
-            network_info, block_device_info)
+            allocations, network_info, block_device_info)
 
         self.vif_driver.plug.assert_called_once_with(
             instance, network_info[0])
@@ -361,6 +361,7 @@ class LXDDriverTest(test.NoDBTestCase):
         image_meta = mock.Mock()
         injected_files = mock.Mock()
         admin_password = mock.Mock()
+        allocations = mock.Mock()
         network_info = [_VIF]
         block_device_info = mock.Mock()
         virtapi = manager.ComputeVirtAPI(mock.MagicMock())
@@ -373,7 +374,7 @@ class LXDDriverTest(test.NoDBTestCase):
             lxdcore_exceptions.LXDAPIException,
             lxd_driver.spawn,
             ctx, instance, image_meta, injected_files, admin_password,
-            network_info, block_device_info)
+            allocations, network_info, block_device_info)
         lxd_driver.cleanup.assert_called_once_with(
             ctx, instance, network_info, block_device_info)
 
@@ -394,6 +395,7 @@ class LXDDriverTest(test.NoDBTestCase):
         image_meta = mock.Mock()
         injected_files = mock.Mock()
         admin_password = mock.Mock()
+        allocations = mock.Mock()
         network_info = [_VIF]
         block_device_info = mock.Mock()
         virtapi = manager.ComputeVirtAPI(mock.MagicMock())
@@ -406,7 +408,7 @@ class LXDDriverTest(test.NoDBTestCase):
             lxdcore_exceptions.LXDAPIException,
             lxd_driver.spawn,
             ctx, instance, image_meta, injected_files, admin_password,
-            network_info, block_device_info)
+            allocations, network_info, block_device_info)
         lxd_driver.cleanup.assert_called_once_with(
             ctx, instance, network_info, block_device_info)
 
@@ -425,6 +427,7 @@ class LXDDriverTest(test.NoDBTestCase):
         image_meta = mock.Mock()
         injected_files = mock.Mock()
         admin_password = mock.Mock()
+        allocations = mock.Mock()
         network_info = [_VIF]
         block_device_info = mock.Mock()
         virtapi = manager.ComputeVirtAPI(mock.MagicMock())
@@ -440,7 +443,7 @@ class LXDDriverTest(test.NoDBTestCase):
             lxdcore_exceptions.LXDAPIException,
             lxd_driver.spawn,
             ctx, instance, image_meta, injected_files, admin_password,
-            network_info, block_device_info)
+            allocations, network_info, block_device_info)
         lxd_driver.cleanup.assert_called_once_with(
             ctx, instance, network_info, block_device_info)
 
@@ -487,13 +490,14 @@ class LXDDriverTest(test.NoDBTestCase):
             image_meta = mock.Mock()
             injected_files = mock.Mock()
             admin_password = mock.Mock()
+            allocations = mock.Mock()
             network_info = [_VIF]
             block_device_info = mock.Mock()
 
             drv.init_host(None)
             drv.spawn(
                 ctx, instance, image_meta, injected_files, admin_password,
-                network_info, block_device_info)
+                allocations, network_info, block_device_info)
 
         test_spawn()
 
@@ -541,6 +545,40 @@ class LXDDriverTest(test.NoDBTestCase):
         mock_container.stop.assert_called_once_with(wait=True)
         mock_container.delete.assert_called_once_with(wait=True)
 
+    def test_destroy_when_in_rescue(self):
+        mock_stopped_container = mock.Mock()
+        mock_stopped_container.status = 'Stopped'
+        mock_rescued_container = mock.Mock()
+        mock_rescued_container.status = 'Running'
+        ctx = context.get_admin_context()
+        instance = fake_instance.fake_instance_obj(
+            ctx, name='test', memory_mb=0)
+        network_info = [_VIF]
+
+        lxd_driver = driver.LXDDriver(None)
+        lxd_driver.init_host(None)
+        lxd_driver.cleanup = mock.Mock()
+
+        # set the vm_state on the fake instance to RESCUED
+        instance.vm_state = vm_states.RESCUED
+
+        # set up the containers.get to return the stopped container and then
+        # the rescued container
+        self.client.containers.get.side_effect = [
+            mock_stopped_container, mock_rescued_container]
+
+        lxd_driver.destroy(ctx, instance, network_info)
+
+        lxd_driver.cleanup.assert_called_once_with(
+            ctx, instance, network_info, None)
+        lxd_driver.client.containers.get.assert_has_calls([
+            mock.call(instance.name),
+            mock.call('{}-rescue'.format(instance.name))])
+        mock_stopped_container.stop.assert_not_called()
+        mock_stopped_container.delete.assert_called_once_with(wait=True)
+        mock_rescued_container.stop.assert_called_once_with(wait=True)
+        mock_rescued_container.delete.assert_called_once_with(wait=True)
+
     def test_destroy_without_instance(self):
         def side_effect(*args, **kwargs):
             raise lxdcore_exceptions.LXDAPIException(MockResponse(404))
