@@ -326,7 +326,7 @@ def _sync_glance_image_to_lxd(client, context, image_ref):
                          'skipping metadata injection...',
                          {'alias': image_ref})
                 with open(image_file, 'rb') as image:
-                    image = client.images.create(image.read(), wait=True)
+                    image = client.images.create(image, wait=True)
             else:
                 metadata = {
                     'architecture': image.get(
@@ -347,7 +347,7 @@ def _sync_glance_image_to_lxd(client, context, image_ref):
                 with open(manifest_file, 'rb') as manifest:
                     with open(image_file, 'rb') as image:
                         image = client.images.create(
-                            image.read(), metadata=manifest.read(),
+                            image, metadata=manifest,
                             wait=True)
 
             image.add_alias(image_ref, '')
@@ -413,7 +413,8 @@ class LXDDriver(driver.ComputeDriver):
         "has_imagecache": False,
         "supports_recreate": False,
         "supports_migrate_to_same_host": False,
-        "supports_attach_interface": True
+        "supports_attach_interface": True,
+        "supports_multiattach": False,
     }
 
     def __init__(self, virtapi):
@@ -466,21 +467,16 @@ class LXDDriver(driver.ComputeDriver):
             raise exception.InstanceNotFound(instance_id=instance.uuid)
 
         state = container.state()
-        mem_kb = state.memory['usage'] >> 10
-        max_mem_kb = state.memory['usage_peak'] >> 10
         return hardware.InstanceInfo(
-            state=_get_power_state(state.status_code),
-            max_mem_kb=max_mem_kb,
-            mem_kb=mem_kb,
-            num_cpu=instance.flavor.vcpus,
-            cpu_time_ns=0)
+            state=_get_power_state(state.status_code))
 
     def list_instances(self):
         """Return a list of all instance names."""
         return [c.name for c in self.client.containers.all()]
 
     def spawn(self, context, instance, image_meta, injected_files,
-              admin_password, network_info=None, block_device_info=None):
+              admin_password, allocations, network_info=None,
+              block_device_info=None):
         """Create a new lxd container as a nova instance.
 
         Creating a new container requires a number of steps. First, the
@@ -614,6 +610,12 @@ class LXDDriver(driver.ComputeDriver):
             if container.status != 'Stopped':
                 container.stop(wait=True)
             container.delete(wait=True)
+            if (instance.vm_state == vm_states.RESCUED):
+                rescued_container = self.client.containers.get(
+                    '%s-rescue' % instance.name)
+                if rescued_container.status != 'Stopped':
+                    rescued_container.stop(wait=True)
+                rescued_container.delete(wait=True)
         except lxd_exceptions.LXDAPIException as e:
             if e.response.status_code == 404:
                 LOG.warning('Failed to delete instance. '
@@ -890,13 +892,15 @@ class LXDDriver(driver.ComputeDriver):
                rescue_password):
         """Rescue a LXD container.
 
-        Rescuing a instance requires a number of steps. First,
-        the failed container is stopped. Next, '-rescue', is
-        appended to the failed container's name, this is done
-        so the container can be unrescued. The container's
-        profile is updated with the rootfs of the
-        failed container. Finally, a new container
-        is created and started.
+        From the perspective of nova, rescuing a instance requires a number of
+        steps. First, the failed container is stopped, and then this method is
+        called.
+
+        So the original container is already stopped, and thus, next,
+        '-rescue', is appended to the failed container's name, this is done so
+        the container can be unrescued. The container's profile is updated with
+        the rootfs of the failed container. Finally, a new container is created
+        and started.
 
         See 'nova.virt.driver.ComputeDriver.rescue` for more
         information.
@@ -1197,9 +1201,40 @@ class LXDDriver(driver.ComputeDriver):
                 format=CONF.config_drive_format)
 
         container = self.client.containers.get(instance.name)
-        container_id_map = container.config[
-            'volatile.last_state.idmap'].split(',')
-        storage_id = container_id_map[2].split(':')[1]
+        storage_id = 0
+        """
+        Determine UID shift used for container uid mapping
+        Sample JSON config from LXD
+        {
+            "volatile.apply_template": "create",
+            ...
+            "volatile.last_state.idmap": "[
+                {
+                \"Isuid\":true,
+                \"Isgid\":false,
+                \"Hostid\":100000,
+                \"Nsid\":0,
+                \"Maprange\":65536
+                },
+                {
+                \"Isuid\":false,
+                \"Isgid\":true,
+                \"Hostid\":100000,
+                \"Nsid\":0,
+                \"Maprange\":65536
+                }] ",
+            "volatile.tap5fd6808a-7b.name": "eth0"
+        }
+        """
+        container_id_map = json.loads(
+            container.config['volatile.last_state.idmap'])
+        uid_map = filter(lambda id_map: id_map.get("Isuid"), container_id_map)
+        if uid_map:
+            storage_id = uid_map[0].get("Hostid", 0)
+        else:
+            # privileged containers does not have uid/gid mapping
+            # LXD API return nothing
+            pass
 
         extra_md = {}
         if admin_password:
@@ -1245,10 +1280,12 @@ class LXDDriver(driver.ComputeDriver):
                 for ent in os.listdir(tmpdir):
                     shutil.copytree(os.path.join(tmpdir, ent),
                                     os.path.join(configdrive_dir, ent))
-                    utils.execute('chmod', '-R', '775', configdrive_dir,
-                                  run_as_root=True)
-                    utils.execute('chown', '-R', storage_id, configdrive_dir,
-                                  run_as_root=True)
+
+                utils.execute('chmod', '-R', '775', configdrive_dir,
+                              run_as_root=True)
+                utils.execute('chown', '-R',
+                              '%s:%s' % (storage_id, storage_id),
+                              configdrive_dir, run_as_root=True)
             finally:
                 if mounted:
                     utils.execute('umount', tmpdir, run_as_root=True)
